#!/usr/bin/env Rscript

library(R0)
library(readr)
library(RCurl)

f1<-"%Y-%m-%d"
f2<-"%m/%d/%Y"

# Make sure you're in the repository containing all the data files

getDataByCountry<-function(data,country){
	indx<-which(data$'Country/Region' == country)
	indx
}

getSI<-function(classification){
	# classification = {"Probable","Certain","all"}
	serial.interval.file<-"serialIntervalData.csv"
	serial.interval<-as.data.frame(read_csv(serial.interval.file))
	serial.interval$ER<-as.integer(serial.interval$ER)
	serial.interval$EL<-as.integer(serial.interval$EL)
	serial.interval$SL<-as.integer(serial.interval$SL)
	serial.interval$SR<-as.integer(serial.interval$SR)
	serial.interval$InfecteeOnset<-as.Date(serial.interval$InfecteeOnset,f2)
	serial.interval$InfectorOnset<-as.Date(serial.interval$InfectorOnset,f2)

	if(classification != "all"){
		indx<-which(serial.interval$SIClassification == classification)
		serial.interval<-serial.interval[indx,]
	}

	serial.interval.tmp<-data.frame(InfectorOnset=serial.interval$InfectorOnset,
									InfecteeOnset=serial.interval$InfecteeOnset)
	SI<-as.vector(serial.interval.tmp$InfecteeOnset - serial.interval.tmp$InfectorOnset)

	SI
}


createTimeSeriesDataCountry<-function(country){
	confirmed.file<-"time_series_19-covid-Confirmed.csv"
	deaths.file<-"time_series_19-covid-Deaths.csv"
	recovered.file<-"time_series_19-covid-Recovered.csv"

	confirmed<-read_csv(confirmed.file)
	deaths<-read_csv(deaths.file)
	recovered<-read_csv(recovered.file)

	indx.conf<-getDataByCountry(confirmed,country)
	indx.death<-getDataByCountry(deaths, country)
	indx.recov<-getDataByCountry(recovered,country)

	len<-length(confirmed)
	data<-data.frame(date=as.Date(names(confirmed[5:length(confirmed)]),"%m/%d/%y"),
						   confirmed=unlist(lapply(seq(5,len,1), function(x) sum(confirmed[indx.conf,x]))),
						   recovered=unlist(lapply(seq(5,len,1), function(x) sum(recovered[indx.recov,x]))),
						   deaths=unlist(lapply(seq(5,len,1), function(x) sum(deaths[indx.death,x])))
						   )
	tmp<-length(data$confirmed)
	TS<-data.frame(date=data$date,
				   incidence=c(0,rev(unlist(lapply(seq(tmp,2,-1) , function(x) data$confirmed[x]-data$confirmed[x-1])))))

	TS
}


getGenerationTime<-function(classification){
	# classification = {"Probable","Certain","all"}
	# For the COVID-19 data, the best fitting GT distribution is always a lognormal distribution
	# Mean and sd vary a little given the type of classification
	SI<-getSI(classification)
	gt<-est.GT(serial.interval=SI)

	# Create a GT class object for the R0 estimation 
	gt.c<-generation.time("lognormal",c(gt$mean,gt$sd))
	gt.c
}

createDataForR0est<-function(country,source){
  # source of data = {"WHO", "CSSE"}
  if(source=="CSSE"){
    TS<-createTimeSeriesDataCountry(country)
  } else if(source=="WHO"){
    TS<-getWHOIncidenceFromCountry(country)
  }

	tmp<-c(TS$incidence)
	names(tmp)<-c(TS$date)
	len<-length(tmp)
	tot<-length(which(tmp !=0))

	# Check that first incidence data is not null, otherwise you'll get an error
	# Get the first index of non-nul incidence data
	# And remove the last incidence data if null 
	first<-(which(tmp != 0)[[1]])
	end<-(which(tmp !=0)[[tot]])
	tmp<-tmp[first:end]

	tmp
}


estimateReTimeDependant<-function(country,classification,source){
	# Estimation of the R(t)
	data<-createDataForR0est(country,source)
	gt<-getGenerationTime(classification)
	start<-as.Date(names(data[1]),f1)
	end<-as.Date(names(data[length(data)]),f1)

	R0.TD<-est.R0.TD(epid=data,
			 	 GT=gt,
		   		 begin=start,
		   		 end=end,
		   		 nsim=5000)

	R0.TD
}

estimateR0ML<-function(country,classification,start,end,source){
	# Here you estimate the R0 using ML method
	# EX: start = "2020-01-24"
	# EX: end = "2020-03-10"
	data<-createDataForR0est(country,source)
	gt<-getGenerationTime(classification)
	R0.ML<-est.R0.ML(epid=data,
				 	  GT=gt,
		   		 	 begin=as.Date(start,f1),
		   		 	 end=as.Date(end,f1),
		   		 	 nsim=5000)

}

getWHOIncidenceFromCountry<-function(country){
  # Data from WHO taken from : https://covid.ourworldindata.org/data/new_cases.csv
  myfile<-getURL("https://covid.ourworldindata.org/data/ecdc/new_cases.csv",ssl.verifyhost=FALSE,ssl.verifypeer=FALSE)
  new.cases<-read.csv(file = textConnection(myfile),header = T)
  #new.cases<-read.csv(file = "covid_new_cases.csv")
  new.cases.country<-new.cases[country]
  new.cases.country[is.na(new.cases.country)]<-0
  names(new.cases.country)<-"incidence"
  dates<-as.vector(new.cases$date)
  len<-length(new.cases.country)
  data<-data.frame(date=as.Date(dates,f1),
                   incidence=new.cases.country)
  data
}


getScale <- function(scale)#Internal scaling function to display proper X-Axis labels
  ###Internal scaling function to display proper X-Axis labels
{
  ##keyword<< internal
  #Scale parameters are used to adjust dates on X-Axis labels
  if (scale == "d") {
    div = 1
  } else if (scale == "w") {
    div = 7
  } else if (scale == "f") {
    div = 15
  } else if (scale == "m") {
    div = 30
  } else {
    stop("Invalid scale parameter.")
  }
  return(div)
}

plotR0TD<-function(x,xscale){
  R = x$R
  epid = x$epid
  epid$t = epid$t[x$begin.nb:(x$end.nb-1)]
  polygon.x = c(epid$t, rev(epid$t))
  polygon.y = c(x$conf.int[1:length(epid$t),1], rev(x$conf.int[1:length(epid$t),2]))
  
  div = get.scale(xscale)
  atLab = pretty(epid$t, n=length(epid$t)/div)
  lab = format(pretty(epid$t, n=length(epid$t)/div),"%d %b")
  
  plot(epid$t, R[1:length(epid$t)], ylim=c(0, max(polygon.y)), xlab="Temps", ylab="R(t)", xaxt="n", pch=NA_integer_, lty="blank", main="Nombre de reproduction efficace")
  polygon(polygon.x, polygon.y, col="gray", border=NA)
  lines(epid$t, R[1:length(epid$t)])
  abline(h=1, lty="dashed", col="gray40")
  
  axis(1, at=atLab, labels=lab)
  
}